#ifndef HAS_CORE
#define HAS_CORE 0
#endif

#if __VERSION__ >= 130

	// For everything MODERN we support, use the "new" generic texture function names.  texture() has no macro
	// arguments because the bias parameter is optional.  We define the old GPU shader4 specific types to generic
	// because we can't macro the other way.
	// This case covers GLSL 130 (various Intel GPUs), GLSL 150 + arb compatibility (modern red and green team
	// drivers on Win/Lin) and GLSL 150 ("core" profile, still experimental)
	#define texture2D					texture
	#define texture2DArray				texture
	#define texture2DProj(a,b)			textureProj(a,b)
	#define texture2DGrad(a,b,c,d)		textureGrad(a,b,c,d)
	#define texture2DLod(a,b,c)			textureLod(a,b,c)
	#define texture2DArrayLod(a,b,c)	textureLod(a,b,c)

	#define shadow2D(a,b)				vec4(texture(a,b))
	#define shadow2DArray(a,b)			vec4(texture(a,b))
	#define shadow2DGrad(a,b,c,d)		vec4(textureGrad(a,b,c,d))
	#define shadow2DArrayGrad(a,b,c,d)	vec4(textureGrad(a,b,c,d))

#elif defined(GL_EXT_gpu_shader4)

	// On OS X legacy profile we're stuck at GLSL 120 + a ton of extensions.  Grab GPU_shader4 - if we don't have
	// that we are missing a TON of stuff we need.  Then cope with the quirk of uint missing.
	// Note that for all practical purposes we expect to ONLY hit this case on OS X.
	#extension GL_EXT_gpu_shader4 : enable
	#define uint unsigned int

#else

	// Back in the day we supported even OLDER skankier stuff, but no need now - every supported GPU
	// we support is at least GPU_shader4 (which matches shader model 4.0, which is in DX10) - because
	// DX10 is a MANDATORY level of hw for XP11.  So we can just pumt out.  This is convenient because
	// GPU_shader4 gets us gradients and texture arrays.
	#error we require GLSL 130 or GLSL 120 + GPU_shader4

#endif

// Opt into a bunch of useful stuff we want.

#if defined(GL_ARB_gpu_shader5)
	#extension GL_ARB_gpu_shader5 : enable
#endif

#if HAS_UBO && !HAS_CORE
	#extension GL_ARB_uniform_buffer_object : enable
#endif

#if HAS_TESS && !HAS_CORE
	#extension GL_ARB_tessellation_shader : enable
#endif


// This is a hack-around for a broken mat3 cast in some version of the GLSL
// compiler on OS X. Tragically I didn't note WHICH version has the bug.  TODO in
// v11: confirm that on our new minimum OS (10.9? 10.10?) this isn't needed.

mat3 mat3_cast(mat4 m)
{ 
	return mat3(m[0].xyz,m[1].xyz,m[2].xyz);
}

#define mat3(X) mat3_cast(X)

// UBO GOO
#if HAS_UBO
#define DEFINE_UBO(x) layout(std140) uniform ubo_##x
#else
#define DEFINE_UBO(x) uniform struct
#endif
